use err_derive::Error;

use super::*;

#[derive(Debug, Error)]
pub enum ParseError {
    #[error(display = "Incomplete input")]
    IncompleteInput,
    #[error(display = "UnexpectedInput at position {}", _0)]
    UnexpectedInput(usize),
    #[error(display = "Duplicate dictionnary key: {}", _0)]
    DuplicateKey(String),
}

#[derive(Debug, Error)]
pub enum TermError {
    #[error(display = "Invalid term index ({:?})", _0)]
    InvalidIndex(ITerm),
    #[error(display = "Wrong type, expected {}, got {}", _0, _1)]
    WrongType(&'static str, &'static str),
    #[error(display = "Wrong length, expected {}, got {}", _0, _1)]
    WrongLength(usize, usize),
    #[error(display = "Wrong dictionnary keys")]
    WrongKeys,
    #[error(display = "Term does not have a raw representation")]
    NoRawRepresentation,
}

#[derive(Debug, Error)]
pub enum ValueError {
    #[error(display = "Invalid term index ({:?})", _0)]
    InvalidIndex(ITerm),
    #[error(display = "Duplicate dictionary key")]
    DuplicateKey,
    #[error(display = "Bad string (contains invalid characters)")]
    BadString,
    #[error(display = "Nested sequence")]
    SeqInSeq,
}
